package src

import (
	"github.com/go-acme/lego/v4/log"
	"github.com/go-acme/lego/v4/platform/config/env"
	"gopkg.in/yaml.v3"
	"os"
)

func ReadConfig() AppConfig {
	InitConfig()
	file, err := os.ReadFile(GetEnvConf().ConfFile)
	if err != nil {
		log.Fatal(err)
	}
	var conf AppConfig
	err = yaml.Unmarshal(file, &conf)
	if err != nil {
		log.Fatal(err)
	}
	log.Infof("读取配置文件成功: %v", conf)
	return conf
}

func InitConfig() {
	// 判断当前目录下是否存在配置文件 config.yml
	_, err := os.Stat(GetEnvConf().ConfFile)

	if os.IsNotExist(err) {
		// 配置文件不存在，则创建一个
		log.Infof("配置文件不存在，自动创建默认配置文件")
		// 生成默认配置
		conf := defaultConf()
		// 创建一个默认的配置文件
		file, err := os.Create(GetEnvConf().ConfFile)
		if err != nil {
			log.Fatal(err)
		}
		data, err := yaml.Marshal(conf)
		if err != nil {
			log.Fatal(err)
		}
		// 写入配置文件
		_, err = file.Write(data)
		if err != nil {
			log.Fatal(err)
		}
	}

}

func defaultConf() *AppConfig {
	return &AppConfig{
		Use:     "Ali",
		CertDir: "cert",
		Provider: AppProvider{
			Ali: AliProvider{
				RegionID:  "cn-hangzhou",
				APIKey:    "api_key",
				SecretKey: "secret_key",
			},
			Tencent: TencentProvider{
				SecretId:  "secret_id",
				SecretKey: "secret_key",
			},
		},
		Domains: []Domain{
			{
				Name:  "example.com",
				Email: "email@example.com",
				Host:  []string{"www.example.com"},
			},
		},
	}
}

type AppConfig struct {
	Use string

	CertDir string

	Provider AppProvider

	Domains []Domain
}

type AppProvider struct {
	Ali     AliProvider
	Tencent TencentProvider
}

type AliProvider struct {
	RegionID  string
	APIKey    string
	SecretKey string
}
type TencentProvider struct {
	SecretId  string
	SecretKey string
}
type Domain struct {
	Name  string
	Email string
	Host  []string
}

const ENV_CONF_FILE = "ACME_MANA_CONF_FILE"

func InitRuntimeConf() EnvConf {
	return EnvConf{
		ConfFile: env.GetOrDefaultString(ENV_CONF_FILE, "config.yml"),
	}
}

type EnvConf struct {
	ConfFile string
}
